@@ -193,6 +193,8 @@ class ScriptMagics(Magics):
             else:
                 raise
         
+        if not cell.endswith('\n'):
+            cell += '\n'
         cell = cell.encode('utf8', 'replace')
         if args.bg:
             self.bg_processes.append(p)
