@@ -24,6 +24,7 @@ Authors:
 # Imports
 #-----------------------------------------------------------------------------
 
+from keyword import iskeyword
 import re
 
 from IPython.core.autocall import IPyAutocall
@@ -80,7 +81,8 @@ def is_shadowed(identifier, ip):
     # This is much safer than calling ofind, which can change state
     return (identifier in ip.user_ns \
             or identifier in ip.user_global_ns \
-            or identifier in ip.ns_table['builtin'])
+            or identifier in ip.ns_table['builtin']\
+            or iskeyword(identifier))
 
 
 #-----------------------------------------------------------------------------
@@ -210,7 +212,7 @@ class PrefilterManager(Configurable):
             self._checkers.remove(checker)
 
     #-------------------------------------------------------------------------
-    # API for managing checkers
+    # API for managing handlers
     #-------------------------------------------------------------------------
 
     def init_handlers(self):
@@ -488,22 +490,6 @@ class AutoMagicChecker(PrefilterChecker):
         return self.prefilter_manager.get_handler_by_name('magic')
 
 
-class AliasChecker(PrefilterChecker):
-
-    priority = Integer(800, config=True)
-
-    def check(self, line_info):
-        "Check if the initital identifier on the line is an alias."
-        # Note: aliases can not contain '.'
-        head = line_info.ifun.split('.',1)[0]
-        if line_info.ifun not in self.shell.alias_manager \
-               or head not in self.shell.alias_manager \
-               or is_shadowed(head, self.shell):
-            return None
-
-        return self.prefilter_manager.get_handler_by_name('alias')
-
-
 class PythonOpsChecker(PrefilterChecker):
 
     priority = Integer(900, config=True)
@@ -591,20 +577,6 @@ class PrefilterHandler(Configurable):
         return "<%s(name=%s)>" % (self.__class__.__name__, self.handler_name)
 
 
-class AliasHandler(PrefilterHandler):
-
-    handler_name = Unicode('alias')
-
-    def handle(self, line_info):
-        """Handle alias input lines. """
-        transformed = self.shell.alias_manager.expand_aliases(line_info.ifun,line_info.the_rest)
-        # pre is needed, because it carries the leading whitespace.  Otherwise
-        # aliases won't work in indented sections.
-        line_out = '%sget_ipython().system(%r)' % (line_info.pre_whitespace, transformed)
-
-        return line_out
-
-
 class MacroHandler(PrefilterHandler):
     handler_name = Unicode("macro")
 
@@ -730,14 +702,12 @@ _default_checkers = [
     IPyAutocallChecker,
     AssignmentChecker,
     AutoMagicChecker,
-    AliasChecker,
     PythonOpsChecker,
     AutocallChecker
 ]
 
 _default_handlers = [
     PrefilterHandler,
-    AliasHandler,
     MacroHandler,
     MagicHandler,
     AutoHandler,
