@@ -1,6 +1,7 @@
-"""
-``autoreload`` is an IPython extension that reloads modules
-automatically before executing the line of code typed.
+"""IPython extension to reload modules before executing user code.
+
+``autoreload`` reloads modules automatically before entering the execution of
+code typed at the IPython prompt.
 
 This makes for example the following workflow possible:
 
@@ -20,8 +21,8 @@ This makes for example the following workflow possible:
    In [6]: some_function()
    Out[6]: 43
 
-The module was reloaded without reloading it explicitly, and the
-object imported with ``from foo import ...`` was also updated.
+The module was reloaded without reloading it explicitly, and the object
+imported with ``from foo import ...`` was also updated.
 
 Usage
 =====
@@ -84,56 +85,68 @@ Some of the known remaining caveats are:
 - Functions that are removed (eg. via monkey-patching) from a module
   before it is reloaded are not upgraded.
 
-- C extension modules cannot be reloaded, and so cannot be
-  autoreloaded.
-
+- C extension modules cannot be reloaded, and so cannot be autoreloaded.
 """
+from __future__ import print_function
 
 skip_doctest = True
 
-# Pauli Virtanen <pav@iki.fi>, 2008.
-# Thomas Heller, 2000.
+#-----------------------------------------------------------------------------
+#  Copyright (C) 2000 Thomas Heller
+#  Copyright (C) 2008 Pauli Virtanen <pav@iki.fi>
+#  Copyright (C) 2012  The IPython Development Team
+#
+#  Distributed under the terms of the BSD License.  The full license is in
+#  the file COPYING, distributed as part of this software.
+#-----------------------------------------------------------------------------
 #
 # This IPython module is written by Pauli Virtanen, based on the autoreload
 # code by Thomas Heller.
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+import os
+import sys
+import traceback
+import types
+import weakref
+
+try:
+    # Reload is not defined by default in Python3.
+    reload
+except NameError:
+    from imp import reload
+
+from IPython.utils import openpy
+from IPython.utils.py3compat import PY3
+
 #------------------------------------------------------------------------------
 # Autoreload functionality
 #------------------------------------------------------------------------------
 
-import time, os, threading, sys, types, imp, inspect, traceback, atexit
-import weakref
-try:
-    reload
-except NameError:
-    from imp import reload
-
-def _get_compiled_ext():
-    """Official way to get the extension of compiled files (.pyc or .pyo)"""
-    for ext, mode, typ in imp.get_suffixes():
-        if typ == imp.PY_COMPILED:
-            return ext
-
-PY_COMPILED_EXT = _get_compiled_ext()
-
 class ModuleReloader(object):
     enabled = False
     """Whether this reloader is enabled"""
 
-    failed = {}
-    """Modules that failed to reload: {module: mtime-on-failed-reload, ...}"""
-
-    modules = {}
-    """Modules specially marked as autoreloadable."""
-
-    skip_modules = {}
-    """Modules specially marked as not autoreloadable."""
-
     check_all = True
     """Autoreload all modules, not just those listed in 'modules'"""
 
-    old_objects = {}
-    """(module-name, name) -> weakref, for replacing old code objects"""
+    def __init__(self):
+        # Modules that failed to reload: {module: mtime-on-failed-reload, ...}
+        self.failed = {}
+        # Modules specially marked as autoreloadable.
+        self.modules = {}
+        # Modules specially marked as not autoreloadable.
+        self.skip_modules = {}
+        # (module-name, name) -> weakref, for replacing old code objects
+        self.old_objects = {}
+        # Module modification timestamps
+        self.modules_mtimes = {}
+
+        # Cache module modification times
+        self.check(check_all=True, do_reload=False)
 
     def mark_module_skipped(self, module_name):
         """Skip reloading the named module in the future"""
@@ -169,16 +182,42 @@ class ModuleReloader(object):
         top_module = sys.modules[top_name]
         return top_module, top_name
 
-    def check(self, check_all=False):
+    def filename_and_mtime(self, module):
+        if not hasattr(module, '__file__'):
+            return None, None
+
+        if module.__name__ == '__main__':
+            # we cannot reload(__main__)
+            return None, None
+
+        filename = module.__file__
+        path, ext = os.path.splitext(filename)
+
+        if ext.lower() == '.py':
+            py_filename = filename
+        else:
+            try:
+                py_filename = openpy.source_from_cache(filename)
+            except ValueError:
+                return None, None
+
+        try:
+            pymtime = os.stat(py_filename).st_mtime
+        except OSError:
+            return None, None
+
+        return py_filename, pymtime
+
+    def check(self, check_all=False, do_reload=True):
         """Check whether some modules need to be reloaded."""
 
         if not self.enabled and not check_all:
             return
 
         if check_all or self.check_all:
-            modules = sys.modules.keys()
+            modules = list(sys.modules.keys())
         else:
-            modules = self.modules.keys()
+            modules = list(self.modules.keys())
 
         for modname in modules:
             m = sys.modules.get(modname, None)
@@ -186,62 +225,58 @@ class ModuleReloader(object):
             if modname in self.skip_modules:
                 continue
 
-            if not hasattr(m, '__file__'):
-                continue
-
-            if m.__name__ == '__main__':
-                # we cannot reload(__main__)
-                continue
-
-            filename = m.__file__
-            path, ext = os.path.splitext(filename)
-
-            if ext.lower() == '.py':
-                ext = PY_COMPILED_EXT
-                pyc_filename = path + PY_COMPILED_EXT
-                py_filename = filename
-            else:
-                pyc_filename = filename
-                py_filename = filename[:-1]
-
-            if ext != PY_COMPILED_EXT:
+            py_filename, pymtime = self.filename_and_mtime(m)
+            if py_filename is None:
                 continue
 
             try:
-                pymtime = os.stat(py_filename).st_mtime
-                if pymtime <= os.stat(pyc_filename).st_mtime:
+                if pymtime <= self.modules_mtimes[modname]:
                     continue
+            except KeyError:
+                self.modules_mtimes[modname] = pymtime
+                continue
+            else:
                 if self.failed.get(py_filename, None) == pymtime:
                     continue
-            except OSError:
-                continue
 
-            try:
-                superreload(m, reload, self.old_objects)
-                if py_filename in self.failed:
-                    del self.failed[py_filename]
-            except:
-                print >> sys.stderr, "[autoreload of %s failed: %s]" % (
-                        modname, traceback.format_exc(1))
-                self.failed[py_filename] = pymtime
+            self.modules_mtimes[modname] = pymtime
+
+            # If we've reached this point, we should try to reload the module
+            if do_reload:
+                try:
+                    superreload(m, reload, self.old_objects)
+                    if py_filename in self.failed:
+                        del self.failed[py_filename]
+                except:
+                    print("[autoreload of %s failed: %s]" % (
+                            modname, traceback.format_exc(1)), file=sys.stderr)
+                    self.failed[py_filename] = pymtime
 
 #------------------------------------------------------------------------------
 # superreload
 #------------------------------------------------------------------------------
 
+if PY3:
+    func_attrs = ['__code__', '__defaults__', '__doc__',
+                  '__closure__', '__globals__', '__dict__']
+else:
+    func_attrs = ['func_code', 'func_defaults', 'func_doc',
+                  'func_closure', 'func_globals', 'func_dict']
+
+
 def update_function(old, new):
     """Upgrade the code object of a function"""
-    for name in ['func_code', 'func_defaults', 'func_doc',
-                 'func_closure', 'func_globals', 'func_dict']:
+    for name in func_attrs:
         try:
             setattr(old, name, getattr(new, name))
         except (AttributeError, TypeError):
             pass
 
+
 def update_class(old, new):
     """Replace stuff in the __dict__ of a class, and upgrade
     method code objects"""
-    for key in old.__dict__.keys():
+    for key in list(old.__dict__.keys()):
         old_obj = getattr(old, key)
 
         try:
@@ -261,28 +296,40 @@ def update_class(old, new):
         except (AttributeError, TypeError):
             pass # skip non-writable attributes
 
+
 def update_property(old, new):
     """Replace get/set/del functions of a property"""
     update_generic(old.fdel, new.fdel)
     update_generic(old.fget, new.fget)
     update_generic(old.fset, new.fset)
 
+
 def isinstance2(a, b, typ):
     return isinstance(a, typ) and isinstance(b, typ)
 
+
 UPDATE_RULES = [
-    (lambda a, b: isinstance2(a, b, types.ClassType),
-     update_class),
-    (lambda a, b: isinstance2(a, b, types.TypeType),
+    (lambda a, b: isinstance2(a, b, type),
      update_class),
     (lambda a, b: isinstance2(a, b, types.FunctionType),
      update_function),
     (lambda a, b: isinstance2(a, b, property),
      update_property),
-    (lambda a, b: isinstance2(a, b, types.MethodType),
-     lambda a, b: update_function(a.im_func, b.im_func)),
 ]
 
+
+if PY3:
+    UPDATE_RULES.extend([(lambda a, b: isinstance2(a, b, types.MethodType),
+                          lambda a, b: update_function(a.__func__, b.__func__)),
+                        ])
+else:
+    UPDATE_RULES.extend([(lambda a, b: isinstance2(a, b, types.ClassType),
+                          update_class),
+                         (lambda a, b: isinstance2(a, b, types.MethodType),
+                          lambda a, b: update_function(a.__func__, b.__func__)),
+                        ])
+
+
 def update_generic(a, b):
     for type_check, update in UPDATE_RULES:
         if type_check(a, b):
@@ -290,12 +337,14 @@ def update_generic(a, b):
             return True
     return False
 
+
 class StrongRef(object):
     def __init__(self, obj):
         self.obj = obj
     def __call__(self):
         return self.obj
 
+
 def superreload(module, reload=reload, old_objects={}):
     """Enhanced version of the builtin reload function.
 
@@ -308,7 +357,7 @@ def superreload(module, reload=reload, old_objects={}):
     """
 
     # collect old objects in the module
-    for name, obj in module.__dict__.items():
+    for name, obj in list(module.__dict__.items()):
         if not hasattr(obj, '__module__') or obj.__module__ != module.__name__:
             continue
         key = (module.__name__, name)
@@ -317,7 +366,7 @@ def superreload(module, reload=reload, old_objects={}):
         except TypeError:
             # weakref doesn't work for all types;
             # create strong references for 'important' cases
-            if isinstance(obj, types.ClassType):
+            if not PY3 and isinstance(obj, types.ClassType):
                 old_objects.setdefault(key, []).append(StrongRef(obj))
 
     # reload module
@@ -327,6 +376,7 @@ def superreload(module, reload=reload, old_objects={}):
         old_name = module.__name__
         module.__dict__.clear()
         module.__dict__['__name__'] = old_name
+        module.__dict__['__loader__'] = old_dict['__loader__']
     except (TypeError, AttributeError, KeyError):
         pass
 
@@ -338,7 +388,7 @@ def superreload(module, reload=reload, old_objects={}):
         raise
 
     # iterate over all objects and update functions & classes
-    for name, new_obj in module.__dict__.items():
+    for name, new_obj in list(module.__dict__.items()):
         key = (module.__name__, name)
         if key not in old_objects: continue
 
@@ -360,16 +410,18 @@ def superreload(module, reload=reload, old_objects={}):
 # IPython connectivity
 #------------------------------------------------------------------------------
 
-from IPython.core.plugin import Plugin
-from IPython.core.hooks import TryNext
+from IPython.core.magic import Magics, magics_class, line_magic
 
-class AutoreloadInterface(object):
+@magics_class
+class AutoreloadMagics(Magics):
     def __init__(self, *a, **kw):
-        super(AutoreloadInterface, self).__init__(*a, **kw)
+        super(AutoreloadMagics, self).__init__(*a, **kw)
         self._reloader = ModuleReloader()
         self._reloader.check_all = False
+        self.loaded_modules = set(sys.modules)
 
-    def magic_autoreload(self, ipself, parameter_s=''):
+    @line_magic
+    def autoreload(self, parameter_s=''):
         r"""%autoreload => Reload modules automatically
 
         %autoreload
@@ -424,7 +476,8 @@ class AutoreloadInterface(object):
             self._reloader.check_all = True
             self._reloader.enabled = True
 
-    def magic_aimport(self, ipself, parameter_s='', stream=None):
+    @line_magic
+    def aimport(self, parameter_s='', stream=None):
         """%aimport => Import modules for automatic reloading.
 
         %aimport
@@ -435,15 +488,11 @@ class AutoreloadInterface(object):
 
         %aimport -foo
         Mark module 'foo' to not be autoreloaded for %autoreload 1
-
         """
-
         modname = parameter_s
         if not modname:
-            to_reload = self._reloader.modules.keys()
-            to_reload.sort()
-            to_skip = self._reloader.skip_modules.keys()
-            to_skip.sort()
+            to_reload = sorted(self._reloader.modules.keys())
+            to_skip = sorted(self._reloader.skip_modules.keys())
             if stream is None:
                 stream = sys.stdout
             if self._reloader.check_all:
@@ -458,30 +507,30 @@ class AutoreloadInterface(object):
             top_module, top_name = self._reloader.aimport_module(modname)
 
             # Inject module to user namespace
-            ipself.push({top_name: top_module})
+            self.shell.push({top_name: top_module})
 
-    def pre_run_code_hook(self, ipself):
-        if not self._reloader.enabled:
-            raise TryNext
-        try:
-            self._reloader.check()
-        except:
-            pass
+    def pre_run_cell(self):
+        if self._reloader.enabled:
+            try:
+                self._reloader.check()
+            except:
+                pass
 
-class AutoreloadPlugin(AutoreloadInterface, Plugin):
-    def __init__(self, shell=None, config=None):
-        super(AutoreloadPlugin, self).__init__(shell=shell, config=config)
+    def post_execute_hook(self):
+        """Cache the modification times of any modules imported in this execution
+        """
+        newly_loaded_modules = set(sys.modules) - self.loaded_modules
+        for modname in newly_loaded_modules:
+            _, pymtime = self._reloader.filename_and_mtime(sys.modules[modname])
+            if pymtime is not None:
+                self._reloader.modules_mtimes[modname] = pymtime
 
-        self.shell.define_magic('autoreload', self.magic_autoreload)
-        self.shell.define_magic('aimport', self.magic_aimport)
-        self.shell.set_hook('pre_run_code_hook', self.pre_run_code_hook)
+        self.loaded_modules.update(newly_loaded_modules)
 
-_loaded = False
 
 def load_ipython_extension(ip):
     """Load the extension in IPython."""
-    global _loaded
-    if not _loaded:
-        plugin = AutoreloadPlugin(shell=ip, config=ip.config)
-        ip.plugin_manager.register_plugin('autoreload', plugin)
-        _loaded = True
+    auto_reload = AutoreloadMagics(ip)
+    ip.register_magics(auto_reload)
+    ip.events.register('pre_run_cell', auto_reload.pre_run_cell)
+    ip.events.register('post_execute', auto_reload.post_execute_hook)
