@@ -8,7 +8,7 @@ except ImportError:
     from distutils.version import LooseVersion as V
 
     if sys.platform != "darwin" or V(platform.mac_ver()[0]) < V("10.9"):
-        from _dummy import *
+        from ._dummy import *
     else:
         from ._nope import *
 
