@@ -1,4 +1,4 @@
 try:
     from jsonschema import *
 except ImportError :
-    from _jsonschema import *
+    from ._jsonschema import *
