@@ -2,4 +2,4 @@ try:
     import pexpect
     from pexpect import *
 except ImportError:
-    from _pexpect import *
+    from ._pexpect import *
