@@ -290,27 +290,36 @@ IPython.utils = (function (IPython) {
     function ansispan(str) {
         // ansispan function adapted from github.com/mmalecki/ansispan (MIT License)
         // regular ansi escapes (using the table above)
+        var is_open = false
         return str.replace(/\033\[(0?[01]|22|39)?([;\d]+)?m/g, function(match, prefix, pattern) {
             if (!pattern) {
                 // [(01|22|39|)m close spans
-                return "</span>";
+                if (is_open) {
+                    is_open = false;
+                    return "</span>";
+                } else {
+                    return "";
+                }
+            } else {
+                is_open = true;
+
+                // consume sequence of color escapes
+                var numbers = pattern.match(/\d+/g);
+                var attrs = {};
+                while (numbers.length > 0) {
+                    _process_numbers(attrs, numbers);
+                }
+
+                var span = "<span ";
+                for (var attr in attrs) {
+                    var value = attrs[attr];
+                    span = span + " " + attr + '="' + attrs[attr] + '"';
+                }
+                return span + ">";
             }
-            // consume sequence of color escapes
-            var numbers = pattern.match(/\d+/g);
-            var attrs = {};
-            while (numbers.length > 0) {
-                _process_numbers(attrs, numbers);
-            }
-            
-            var span = "<span ";
-            for (var attr in attrs) {
-                var value = attrs[attr];
-                span = span + " " + attr + '="' + attrs[attr] + '"';
-            }
-            return span + ">";
         });
     };
-    
+
     // Transform ANSI color escape codes into HTML <span> tags with css
     // classes listed in the above ansi_colormap object. The actual color used
     // are set in the css file.
@@ -487,6 +496,23 @@ IPython.utils = (function (IPython) {
             return false;
         }
     }
+    
+    var ajax_error_msg = function (jqXHR) {
+        // Return a JSON error message if there is one,
+        // otherwise the basic HTTP status text.
+        if (jqXHR.responseJSON && jqXHR.responseJSON.message) {
+            return jqXHR.responseJSON.message;
+        } else {
+            return jqXHR.statusText;
+        }
+    }
+    var log_ajax_error = function (jqXHR, status, error) {
+        // log ajax failures with informative messages
+        var msg = "API request failed (" + jqXHR.status + "): ";
+        console.log(jqXHR);
+        msg += ajax_error_msg(jqXHR);
+        console.log(msg);
+    };
 
     return {
         regex_split : regex_split,
@@ -506,7 +532,9 @@ IPython.utils = (function (IPython) {
         browser : browser,
         platform: platform,
         is_or_has : is_or_has,
-        is_focused : is_focused
+        is_focused : is_focused,
+        ajax_error_msg : ajax_error_msg,
+        log_ajax_error : log_ajax_error,
     };
 
 }(IPython));
