@@ -280,9 +280,13 @@ var IPython = (function (IPython) {
             IPython.notebook.restart_kernel();
         });
         // Help
-        this.element.find('#notebook_tour').click(function () {
-            IPython.tour.start();
-        });
+        if (IPython.tour) {
+            this.element.find('#notebook_tour').click(function () {
+                IPython.tour.start();
+            });
+        } else {
+            this.element.find('#notebook_tour').addClass("disabled");
+        }
         this.element.find('#keyboard_shortcuts').click(function () {
             IPython.quick_help.show_keyboard_shortcuts();
         });
