@@ -14,8 +14,6 @@ Authors
 
 import nose.tools as nt
 
-from IPython.testing import decorators as dec
-
 from IPython.kernel import launcher, connect
 from IPython import kernel
 
@@ -23,25 +21,21 @@ from IPython import kernel
 # Classes and functions
 #-----------------------------------------------------------------------------
 
-@dec.parametric
 def test_kms():
     for base in ("", "Multi"):
         KM = base + "KernelManager"
-        yield nt.assert_true(KM in dir(kernel), KM)
+        nt.assert_in(KM, dir(kernel))
 
-@dec.parametric
 def test_kcs():
     for base in ("", "Blocking"):
         KM = base + "KernelClient"
-        yield nt.assert_true(KM in dir(kernel), KM)
+        nt.assert_in(KM, dir(kernel))
 
-@dec.parametric
 def test_launcher():
     for name in launcher.__all__:
-        yield nt.assert_true(name in dir(kernel), name)
+        nt.assert_in(name, dir(kernel))
 
-@dec.parametric
 def test_connect():
     for name in connect.__all__:
-        yield nt.assert_true(name in dir(kernel), name)
+        nt.assert_in(name, dir(kernel))
 
