@@ -134,8 +134,8 @@ def latex_to_png_dvipng(s, wrap):
 
         with open(os.devnull, 'w') as devnull:
             subprocess.check_call(
-                ["latex", "-halt-on-error", tmpfile], cwd=workdir,
-                stdout=devnull, stderr=devnull)
+                ["latex", "-halt-on-error", "-interaction", "batchmode", tmpfile],
+                cwd=workdir, stdout=devnull, stderr=devnull)
 
             subprocess.check_call(
                 ["dvipng", "-T", "tight", "-x", "1500", "-z", "9",
@@ -143,10 +143,9 @@ def latex_to_png_dvipng(s, wrap):
                 stdout=devnull, stderr=devnull)
 
         with open(outfile, "rb") as f:
-            bin_data = f.read()
+            return f.read()
     finally:
         shutil.rmtree(workdir)
-    return bin_data
 
 
 def kpsewhich(filename):
