@@ -20,8 +20,6 @@ Authors:
 # Imports
 #-----------------------------------------------------------------------------
 
-from __future__ import with_statement
-
 import os
 import logging
 import re
@@ -38,8 +36,10 @@ from IPython.core.application import (
     base_flags as base_ip_flags
 )
 from IPython.utils.path import expand_path
+from IPython.utils import py3compat
+from IPython.utils.py3compat import unicode_type
 
-from IPython.utils.traitlets import Unicode, Bool, Instance, Dict, List
+from IPython.utils.traitlets import Unicode, Bool, Instance, Dict
 
 #-----------------------------------------------------------------------------
 # Module errors
@@ -106,11 +106,11 @@ class BaseParallelApplication(BaseIPythonApplication):
         """override default log format to include time"""
         return u"%(asctime)s.%(msecs).03d [%(name)s]%(highlevel)s %(message)s"
 
-    work_dir = Unicode(os.getcwdu(), config=True,
+    work_dir = Unicode(py3compat.getcwd(), config=True,
         help='Set the working dir for the process.'
     )
     def _work_dir_changed(self, name, old, new):
-        self.work_dir = unicode(expand_path(new))
+        self.work_dir = unicode_type(expand_path(new))
 
     log_to_file = Bool(config=True,
         help="whether to log to a file")
@@ -157,7 +157,7 @@ class BaseParallelApplication(BaseIPythonApplication):
         
     def to_work_dir(self):
         wd = self.work_dir
-        if unicode(wd) != os.getcwdu():
+        if unicode_type(wd) != py3compat.getcwd():
             os.chdir(wd)
             self.log.info("Changing to working dir: %s" % wd)
         # This is the working dir by now.
@@ -272,5 +272,5 @@ class BaseParallelApplication(BaseIPythonApplication):
                     " Making the likely assumption that it is."%pid
                 )
                 return True
-            pids = map(int, re.findall(br'^\W*\d+', output, re.MULTILINE))
+            pids = list(map(int, re.findall(br'^\W*\d+', output, re.MULTILINE)))
             return pid in pids
