@@ -4,6 +4,7 @@ Authors:
 
 * MinRK
 """
+from __future__ import print_function
 #-----------------------------------------------------------------------------
 #  Copyright (C) 2010-2011  The IPython Development Team
 #
@@ -34,11 +35,12 @@ from IPython.config.configurable import MultipleInstanceError
 from IPython.core.application import BaseIPythonApplication
 from IPython.core.profiledir import ProfileDir, ProfileDirError
 
+from IPython.utils.capture import RichOutput
 from IPython.utils.coloransi import TermColors
-from IPython.utils.jsonutil import rekey
-from IPython.utils.localinterfaces import LOCALHOST, LOCAL_IPS
+from IPython.utils.jsonutil import rekey, extract_dates, parse_date
+from IPython.utils.localinterfaces import localhost, is_local_ip
 from IPython.utils.path import get_ipython_dir
-from IPython.utils.py3compat import cast_bytes
+from IPython.utils.py3compat import cast_bytes, string_types, xrange, iteritems
 from IPython.utils.traitlets import (HasTraits, Integer, Instance, Unicode,
                                     Dict, List, Bool, Set, Any)
 from IPython.external.decorator import decorator
@@ -54,11 +56,6 @@ from IPython.kernel.zmq import serialize
 from .asyncresult import AsyncResult, AsyncHubResult
 from .view import DirectView, LoadBalancedView
 
-if sys.version_info[0] >= 3:
-    # xrange is used in a couple 'isinstance' tests in py2
-    # should be just 'range' in 3k
-    xrange = range
-
 #--------------------------------------------------------------------------
 # Decorators for Client methods
 #--------------------------------------------------------------------------
@@ -75,7 +72,7 @@ def spin_first(f, self, *args, **kwargs):
 #--------------------------------------------------------------------------
 
 
-class ExecuteReply(object):
+class ExecuteReply(RichOutput):
     """wrapper for finished Execute results"""
     def __init__(self, msg_id, content, metadata):
         self.msg_id = msg_id
@@ -83,6 +80,39 @@ class ExecuteReply(object):
         self.execution_count = content['execution_count']
         self.metadata = metadata
     
+    # RichOutput overrides
+    
+    @property
+    def source(self):
+        pyout = self.metadata['pyout']
+        if pyout:
+            return pyout.get('source', '')
+    
+    @property
+    def data(self):
+        pyout = self.metadata['pyout']
+        if pyout:
+            return pyout.get('data', {})
+    
+    @property
+    def _metadata(self):
+        pyout = self.metadata['pyout']
+        if pyout:
+            return pyout.get('metadata', {})
+    
+    def display(self):
+        from IPython.display import publish_display_data
+        publish_display_data(self.source, self.data, self.metadata)
+    
+    def _repr_mime_(self, mime):
+        if mime not in self.data:
+            return
+        data = self.data[mime]
+        if mime in self._metadata:
+            return data, self._metadata[mime]
+        else:
+            return data
+    
     def __getitem__(self, key):
         return self.metadata[key]
     
@@ -128,34 +158,6 @@ class ExecuteReply(object):
                 self.metadata['engine_id'], self.execution_count
             ) + normal + text_out
         )
-    
-    def _repr_html_(self):
-        pyout = self.metadata['pyout'] or {'data':{}}
-        return pyout['data'].get("text/html")
-    
-    def _repr_latex_(self):
-        pyout = self.metadata['pyout'] or {'data':{}}
-        return pyout['data'].get("text/latex")
-    
-    def _repr_json_(self):
-        pyout = self.metadata['pyout'] or {'data':{}}
-        return pyout['data'].get("application/json")
-    
-    def _repr_javascript_(self):
-        pyout = self.metadata['pyout'] or {'data':{}}
-        return pyout['data'].get("application/javascript")
-    
-    def _repr_png_(self):
-        pyout = self.metadata['pyout'] or {'data':{}}
-        return pyout['data'].get("image/png")
-    
-    def _repr_jpeg_(self):
-        pyout = self.metadata['pyout'] or {'data':{}}
-        return pyout['data'].get("image/jpeg")
-    
-    def _repr_svg_(self):
-        pyout = self.metadata['pyout'] or {'data':{}}
-        return pyout['data'].get("image/svg+xml")
 
 
 class Metadata(dict):
@@ -193,21 +195,21 @@ class Metadata(dict):
 
     def __getattr__(self, key):
         """getattr aliased to getitem"""
-        if key in self.iterkeys():
+        if key in self:
             return self[key]
         else:
             raise AttributeError(key)
 
     def __setattr__(self, key, value):
         """setattr aliased to setitem, with strict"""
-        if key in self.iterkeys():
+        if key in self:
             self[key] = value
         else:
             raise AttributeError(key)
 
     def __setitem__(self, key, value):
         """strict static key enforcement"""
-        if key in self.iterkeys():
+        if key in self:
             dict.__setitem__(self, key, value)
         else:
             raise KeyError(key)
@@ -427,13 +429,13 @@ class Client(HasTraits):
         
         url = cfg['registration']
         
-        if location is not None and addr == LOCALHOST:
+        if location is not None and addr == localhost():
             # location specified, and connection is expected to be local
-            if location not in LOCAL_IPS and not sshserver:
+            if not is_local_ip(location) and not sshserver:
                 # load ssh from JSON *only* if the controller is not on
                 # this machine
                 sshserver=cfg['ssh']
-            if location not in LOCAL_IPS and not sshserver:
+            if not is_local_ip(location) and not sshserver:
                 # warn if no ssh specified, but SSH is probably needed
                 # This is only a warning, because the most likely cause
                 # is a local Controller on a laptop whose IP is dynamic
@@ -489,7 +491,7 @@ class Client(HasTraits):
                                     }
         self._queue_handlers = {'execute_reply' : self._handle_execute_reply,
                                 'apply_reply' : self._handle_apply_reply}
-
+        
         try:
             self._connect(sshserver, ssh_kwargs, timeout)
         except:
@@ -532,13 +534,13 @@ class Client(HasTraits):
 
     def _update_engines(self, engines):
         """Update our engines dict and _ids from a dict of the form: {id:uuid}."""
-        for k,v in engines.iteritems():
+        for k,v in iteritems(engines):
             eid = int(k)
             if eid not in self._engines:
                 self._ids.append(eid)
             self._engines[eid] = v
         self._ids = sorted(self._ids)
-        if sorted(self._engines.keys()) != range(len(self._engines)) and \
+        if sorted(self._engines.keys()) != list(range(len(self._engines))) and \
                         self._task_scheme == 'pure' and self._task_socket:
             self._stop_scheduling_tasks()
 
@@ -566,7 +568,7 @@ class Client(HasTraits):
 
         if targets is None:
             targets = self._ids
-        elif isinstance(targets, basestring):
+        elif isinstance(targets, string_types):
             if targets.lower() == 'all':
                 targets = self._ids
             else:
@@ -579,7 +581,7 @@ class Client(HasTraits):
             targets = [targets]
 
         if isinstance(targets, slice):
-            indices = range(len(self._ids))[targets]
+            indices = list(range(len(self._ids))[targets])
             ids = self.ids
             targets = [ ids[i] for i in indices ]
 
@@ -673,7 +675,7 @@ class Client(HasTraits):
         if 'date' in parent:
             md['submitted'] = parent['date']
         if 'started' in msg_meta:
-            md['started'] = msg_meta['started']
+            md['started'] = parse_date(msg_meta['started'])
         if 'date' in header:
             md['completed'] = header['date']
         return md
@@ -732,9 +734,9 @@ class Client(HasTraits):
         msg_id = parent['msg_id']
         if msg_id not in self.outstanding:
             if msg_id in self.history:
-                print ("got stale result: %s"%msg_id)
+                print("got stale result: %s"%msg_id)
             else:
-                print ("got unknown result: %s"%msg_id)
+                print("got unknown result: %s"%msg_id)
         else:
             self.outstanding.remove(msg_id)
 
@@ -768,11 +770,11 @@ class Client(HasTraits):
         msg_id = parent['msg_id']
         if msg_id not in self.outstanding:
             if msg_id in self.history:
-                print ("got stale result: %s"%msg_id)
-                print self.results[msg_id]
-                print msg
+                print("got stale result: %s"%msg_id)
+                print(self.results[msg_id])
+                print(msg)
             else:
-                print ("got unknown result: %s"%msg_id)
+                print("got unknown result: %s"%msg_id)
         else:
             self.outstanding.remove(msg_id)
         content = msg['content']
@@ -920,6 +922,15 @@ class Client(HasTraits):
         else:
             return self.direct_view(key)
 
+    def __iter__(self):
+        """Since we define getitem, Client is iterable
+
+        but unless we also define __iter__, it won't work correctly unless engine IDs
+        start at zero and are continuous.
+        """
+        for eid in self.ids:
+            yield self.direct_view(eid)
+
     #--------------------------------------------------------------------------
     # Begin public methods
     #--------------------------------------------------------------------------
@@ -956,7 +967,7 @@ class Client(HasTraits):
 
     def close(self, linger=None):
         """Close my zmq Sockets
-
+        
         If `linger`, set the zmq LINGER socket option,
         which allows discarding of messages.
         """
@@ -1060,7 +1071,7 @@ class Client(HasTraits):
         if jobs is None:
             theids = self.outstanding
         else:
-            if isinstance(jobs, (int, basestring, AsyncResult)):
+            if isinstance(jobs, string_types + (int, AsyncResult)):
                 jobs = [jobs]
             theids = set()
             for job in jobs:
@@ -1068,7 +1079,7 @@ class Client(HasTraits):
                     # index access
                     job = self.history[job]
                 elif isinstance(job, AsyncResult):
-                    map(theids.add, job.msg_ids)
+                    theids.update(job.msg_ids)
                     continue
                 theids.add(job)
         if not theids.intersection(self.outstanding):
@@ -1128,9 +1139,9 @@ class Client(HasTraits):
         targets = self._build_targets(targets)[0]
         
         msg_ids = []
-        if isinstance(jobs, (basestring,AsyncResult)):
+        if isinstance(jobs, string_types + (AsyncResult,)):
             jobs = [jobs]
-        bad_ids = filter(lambda obj: not isinstance(obj, (basestring, AsyncResult)), jobs)
+        bad_ids = [obj for obj in jobs if not isinstance(obj, string_types + (AsyncResult,))]
         if bad_ids:
             raise TypeError("Invalid msg_id type %r, expected str or AsyncResult"%bad_ids[0])
         for j in jobs:
@@ -1281,7 +1292,7 @@ class Client(HasTraits):
         metadata = metadata if metadata is not None else {}
 
         # validate arguments
-        if not isinstance(code, basestring):
+        if not isinstance(code, string_types):
             raise TypeError("code must be text, not %s" % type(code))
         if not isinstance(metadata, dict):
             raise TypeError("metadata must be dict, not %s" % type(metadata))
@@ -1409,12 +1420,12 @@ class Client(HasTraits):
         for id in indices_or_msg_ids:
             if isinstance(id, int):
                 id = self.history[id]
-            if not isinstance(id, basestring):
+            if not isinstance(id, string_types):
                 raise TypeError("indices must be str or int, not %r"%id)
             theids.append(id)
 
-        local_ids = filter(lambda msg_id: msg_id in self.outstanding or msg_id in self.results, theids)
-        remote_ids = filter(lambda msg_id: msg_id not in local_ids, theids)
+        local_ids = [msg_id for msg_id in theids if (msg_id in self.outstanding or msg_id in self.results)]
+        remote_ids = [msg_id for msg_id in theids if msg_id not in local_ids]
         
         # given single msg_id initially, get_result shot get the result itself,
         # not a length-one list
@@ -1464,7 +1475,7 @@ class Client(HasTraits):
         for id in indices_or_msg_ids:
             if isinstance(id, int):
                 id = self.history[id]
-            if not isinstance(id, basestring):
+            if not isinstance(id, string_types):
                 raise TypeError("indices must be str or int, not %r"%id)
             theids.append(id)
 
@@ -1521,7 +1532,7 @@ class Client(HasTraits):
         for msg_id in msg_ids:
             if isinstance(msg_id, int):
                 msg_id = self.history[msg_id]
-            if not isinstance(msg_id, basestring):
+            if not isinstance(msg_id, string_types):
                 raise TypeError("msg_ids must be str, not %r"%msg_id)
             theids.append(msg_id)
 
@@ -1562,8 +1573,8 @@ class Client(HasTraits):
         for msg_id in sorted(theids):
             if msg_id in content['completed']:
                 rec = content[msg_id]
-                parent = rec['header']
-                header = rec['result_header']
+                parent = extract_dates(rec['header'])
+                header = extract_dates(rec['result_header'])
                 rcontent = rec['result_content']
                 iodict = rec['io']
                 if isinstance(rcontent, str):
@@ -1578,7 +1589,7 @@ class Client(HasTraits):
                 )
                 md.update(self._extract_metadata(md_msg))
                 if rec.get('received'):
-                    md['received'] = rec['received']
+                    md['received'] = parse_date(rec['received'])
                 md.update(iodict)
                 
                 if rcontent['status'] == 'ok':
@@ -1639,16 +1650,16 @@ class Client(HasTraits):
         if not targets: # needed as _build_targets otherwise uses all engines
             return []
         target_ids = self._build_targets(targets)[0] 
-        return filter(lambda md_id: self.metadata[md_id]["engine_uuid"] in target_ids, self.metadata)
+        return [md_id for md_id in self.metadata if self.metadata[md_id]["engine_uuid"] in target_ids]
     
     def _build_msgids_from_jobs(self, jobs=None):
         """Build a list of msg_ids from "jobs" """
         if not jobs:
             return []
         msg_ids = []
-        if isinstance(jobs, (basestring,AsyncResult)):
+        if isinstance(jobs, string_types + (AsyncResult,)):
             jobs = [jobs]
-        bad_ids = filter(lambda obj: not isinstance(obj, (basestring, AsyncResult)), jobs)
+        bad_ids = [obj for obj in jobs if not isinstance(obj, string_types + (AsyncResult,))]
         if bad_ids:
             raise TypeError("Invalid msg_id type %r, expected str or AsyncResult"%bad_ids[0])
         for j in jobs:
@@ -1690,7 +1701,7 @@ class Client(HasTraits):
         """
         if not targets and not jobs:
             raise ValueError("Must specify at least one of `targets` and `jobs`")
-
+        
         if jobs == 'all':
             if self.outstanding:
                 raise RuntimeError("Can't purge outstanding tasks: %s" % self.outstanding)
@@ -1703,8 +1714,9 @@ class Client(HasTraits):
             still_outstanding = self.outstanding.intersection(msg_ids)
             if still_outstanding:
                 raise RuntimeError("Can't purge outstanding tasks: %s" % still_outstanding)
-            map(self.results.pop, msg_ids)
-            map(self.metadata.pop, msg_ids)
+            for mid in msg_ids:
+                self.results.pop(mid)
+                self.metadata.pop(mid)
 
 
     @spin_first
@@ -1824,7 +1836,7 @@ class Client(HasTraits):
             The subset of keys to be returned.  The default is to fetch everything but buffers.
             'msg_id' will *always* be included.
         """
-        if isinstance(keys, basestring):
+        if isinstance(keys, string_types):
             keys = [keys]
         content = dict(query=query, keys=keys)
         self.session.send(self._query_socket, "db_request", content=content)
@@ -1843,6 +1855,13 @@ class Client(HasTraits):
         has_bufs = buffer_lens is not None
         has_rbufs = result_buffer_lens is not None
         for i,rec in enumerate(records):
+            # unpack datetime objects
+            for hkey in ('header', 'result_header'):
+                if hkey in rec:
+                    rec[hkey] = extract_dates(rec[hkey])
+            for dtkey in ('submitted', 'started', 'completed', 'received'):
+                if dtkey in rec:
+                    rec[dtkey] = parse_date(rec[dtkey])
             # relink buffers
             if has_bufs:
                 blen = buffer_lens[i]
