@@ -52,7 +52,7 @@ from .dependency import Dependency
 @decorator
 def logged(f,self,*args,**kwargs):
     # print ("#--------------------")
-    self.log.debug("scheduler::%s(*%s,**%s)", f.func_name, args, kwargs)
+    self.log.debug("scheduler::%s(*%s,**%s)", f.__name__, args, kwargs)
     # print ("#--")
     return f(self,*args, **kwargs)
 
@@ -140,9 +140,10 @@ class Job(object):
         self.after = after
         self.follow = follow
         self.timeout = timeout
-        self.removed = False # used for lazy-delete from sorted queue
         
+        self.removed = False # used for lazy-delete from sorted queue
         self.timestamp = time.time()
+        self.timeout_id = 0
         self.blacklist = set()
 
     def __lt__(self, other):
@@ -155,6 +156,7 @@ class Job(object):
     def dependents(self):
         return self.follow.union(self.after)
 
+
 class TaskScheduler(SessionFactory):
     """Python TaskScheduler object.
 
@@ -362,7 +364,7 @@ class TaskScheduler(SessionFactory):
                 date=datetime.now(),
             )
             msg = self.session.msg('apply_reply', content, parent=parent, metadata=md)
-            raw_reply = map(zmq.Message, self.session.serialize(msg, ident=idents))
+            raw_reply = list(map(zmq.Message, self.session.serialize(msg, ident=idents)))
             # and dispatch it
             self.dispatch_result(raw_reply)
 
@@ -400,8 +402,7 @@ class TaskScheduler(SessionFactory):
         # get targets as a set of bytes objects
         # from a list of unicode objects
         targets = md.get('targets', [])
-        targets = map(cast_bytes, targets)
-        targets = set(targets)
+        targets = set(map(cast_bytes, targets))
 
         retries = md.get('retries', 0)
         self.retries[msg_id] = retries
@@ -433,19 +434,14 @@ class TaskScheduler(SessionFactory):
         # location dependencies
         follow = Dependency(md.get('follow', []))
 
-        # turn timeouts into datetime objects:
         timeout = md.get('timeout', None)
         if timeout:
-            timeout = time.time() + float(timeout)
+            timeout = float(timeout)
 
         job = Job(msg_id=msg_id, raw_msg=raw_msg, idents=idents, msg=msg,
                  header=header, targets=targets, after=after, follow=follow,
                  timeout=timeout, metadata=md,
         )
-        if timeout:
-            # schedule timeout callback
-            self.loop.add_timeout(timeout, lambda : self.job_timeout(job))
-        
         # validate and reduce dependencies:
         for dep in after,follow:
             if not dep: # empty dependency
@@ -469,11 +465,14 @@ class TaskScheduler(SessionFactory):
         else:
             self.save_unmet(job)
 
-    def job_timeout(self, job):
+    def job_timeout(self, job, timeout_id):
         """callback for a job's timeout.
         
         The job may or may not have been run at this point.
         """
+        if job.timeout_id != timeout_id:
+            # not the most recent call
+            return
         now = time.time()
         if job.timeout >= (now + 1):
             self.log.warn("task %s timeout fired prematurely: %s > %s",
@@ -515,7 +514,7 @@ class TaskScheduler(SessionFactory):
     def available_engines(self):
         """return a list of available engine indices based on HWM"""
         if not self.hwm:
-            return range(len(self.targets))
+            return list(range(len(self.targets)))
         available = []
         for idx in range(len(self.targets)):
             if self.loads[idx] < self.hwm:
@@ -547,7 +546,7 @@ class TaskScheduler(SessionFactory):
                 # check follow
                 return job.follow.check(self.completed[target], self.failed[target])
 
-            indices = filter(can_run, available)
+            indices = list(filter(can_run, available))
 
             if not indices:
                 # couldn't run
@@ -590,6 +589,14 @@ class TaskScheduler(SessionFactory):
             if dep_id not in self.graph:
                 self.graph[dep_id] = set()
             self.graph[dep_id].add(msg_id)
+        
+        # schedule timeout callback
+        if job.timeout:
+            timeout_id = job.timeout_id = job.timeout_id + 1
+            self.loop.add_timeout(time.time() + job.timeout,
+                lambda : self.job_timeout(job, timeout_id)
+            )
+        
 
     def submit_task(self, job, indices=None):
         """Submit a task to any of a subset of our targets."""
@@ -633,7 +640,7 @@ class TaskScheduler(SessionFactory):
             else:
                 self.finish_job(idx)
         except Exception:
-            self.log.error("task::Invaid result: %r", raw_msg, exc_info=True)
+            self.log.error("task::Invalid result: %r", raw_msg, exc_info=True)
             return
 
         md = msg['metadata']
