@@ -6,29 +6,30 @@ import sys
 import uuid
 
 # System library imports
-from pygments.lexers import PythonLexer
 from IPython.external import qt
 from IPython.external.qt import QtCore, QtGui
+from IPython.utils import py3compat
+from IPython.utils.importstring import import_item
 
 # Local imports
 from IPython.core.inputsplitter import InputSplitter, IPythonInputSplitter
 from IPython.core.inputtransformer import classic_prompt
 from IPython.core.oinspect import call_tip
 from IPython.qt.base_frontend_mixin import BaseFrontendMixin
-from IPython.utils.traitlets import Bool, Instance, Unicode
-from bracket_matcher import BracketMatcher
-from call_tip_widget import CallTipWidget
-from completion_lexer import CompletionLexer
-from history_console_widget import HistoryConsoleWidget
-from pygments_highlighter import PygmentsHighlighter
+from IPython.utils.traitlets import Any, Bool, Instance, Unicode, DottedObjectName
+from .bracket_matcher import BracketMatcher
+from .call_tip_widget import CallTipWidget
+from .completion_lexer import CompletionLexer
+from .history_console_widget import HistoryConsoleWidget
+from .pygments_highlighter import PygmentsHighlighter
 
 
 class FrontendHighlighter(PygmentsHighlighter):
     """ A PygmentsHighlighter that understands and ignores prompts.
     """
 
-    def __init__(self, frontend):
-        super(FrontendHighlighter, self).__init__(frontend._control.document())
+    def __init__(self, frontend, lexer=None):
+        super(FrontendHighlighter, self).__init__(frontend._control.document(), lexer=lexer)
         self._current_offset = 0
         self._frontend = frontend
         self.highlighting_on = False
@@ -100,6 +101,24 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
 
     confirm_restart = Bool(True, config=True,
         help="Whether to ask for user confirmation when restarting kernel")
+    
+    lexer_class = DottedObjectName(config=True,
+        help="The pygments lexer class to use."
+    )
+    def _lexer_class_changed(self, name, old, new):
+        lexer_class = import_item(new)
+        self.lexer = lexer_class()
+    
+    def _lexer_class_default(self):
+        if py3compat.PY3:
+            return 'pygments.lexers.Python3Lexer'
+        else:
+            return 'pygments.lexers.PythonLexer'
+    
+    lexer = Any()
+    def _lexer_default(self):
+        lexer_class = import_item(self.lexer_class)
+        return lexer_class()
 
     # Emitted when a user visible 'execute_request' has been submitted to the
     # kernel from the FrontendWidget. Contains the code to be executed.
@@ -141,10 +160,10 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         # FrontendWidget protected variables.
         self._bracket_matcher = BracketMatcher(self._control)
         self._call_tip_widget = CallTipWidget(self._control)
-        self._completion_lexer = CompletionLexer(PythonLexer())
+        self._completion_lexer = CompletionLexer(self.lexer)
         self._copy_raw_action = QtGui.QAction('Copy (Raw Text)', None)
         self._hidden = False
-        self._highlighter = FrontendHighlighter(self)
+        self._highlighter = FrontendHighlighter(self, lexer=self.lexer)
         self._input_splitter = self._input_splitter_class()
         self._kernel_manager = None
         self._kernel_client = None
@@ -178,6 +197,9 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         self._local_kernel = kw.get('local_kernel',
                                     FrontendWidget._local_kernel)
 
+        # Whether or not a clear_output call is pending new output.
+        self._pending_clearoutput = False
+
     #---------------------------------------------------------------------------
     # 'ConsoleWidget' public interface
     #---------------------------------------------------------------------------
@@ -205,7 +227,10 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
             'interactive' is True; otherwise, it is False.
         """
         self._input_splitter.reset()
-        complete = self._input_splitter.push(source)
+        try:
+            complete = self._input_splitter.push(source)
+        except SyntaxError:
+            return True
         if interactive:
             complete = not self._input_splitter.push_accepts_more()
         return complete
@@ -317,6 +342,14 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     #---------------------------------------------------------------------------
     # 'BaseFrontendMixin' abstract interface
     #---------------------------------------------------------------------------
+    def _handle_clear_output(self, msg):
+        """Handle clear output messages."""
+        if not self._hidden and self._is_from_this_session(msg):
+            wait = msg['content'].get('wait', True)
+            if wait:
+                self._pending_clearoutput = True
+            else:
+                self.clear_output()
 
     def _handle_complete_reply(self, rep):
         """ Handle replies for tab completion.
@@ -498,6 +531,7 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         """
         self.log.debug("pyout: %s", msg.get('content', ''))
         if not self._hidden and self._is_from_this_session(msg):
+            self.flush_clearoutput()
             text = msg['content']['data']
             self._append_plain_text(text + '\n', before_prompt=True)
 
@@ -506,18 +540,13 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         """
         self.log.debug("stream: %s", msg.get('content', ''))
         if not self._hidden and self._is_from_this_session(msg):
-            # Most consoles treat tabs as being 8 space characters. Convert tabs
-            # to spaces so that output looks as expected regardless of this
-            # widget's tab width.
-            text = msg['content']['data'].expandtabs(8)
-
-            self._append_plain_text(text, before_prompt=True)
-            self._control.moveCursor(QtGui.QTextCursor.End)
+            self.flush_clearoutput()
+            self.append_stream(msg['content']['data'])
 
     def _handle_shutdown_reply(self, msg):
         """ Handle shutdown signal, only if from other console.
         """
-        self.log.warn("shutdown: %s", msg.get('content', ''))
+        self.log.info("shutdown: %s", msg.get('content', ''))
         restart = msg.get('content', {}).get('restart', False)
         if not self._hidden and not self._is_from_this_session(msg):
             # got shutdown reply, request came from session other than ours
@@ -663,6 +692,29 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
                 before_prompt=True
             )
 
+    def append_stream(self, text):
+        """Appends text to the output stream."""
+        # Most consoles treat tabs as being 8 space characters. Convert tabs
+        # to spaces so that output looks as expected regardless of this
+        # widget's tab width.
+        text = text.expandtabs(8)
+        self._append_plain_text(text, before_prompt=True)
+        self._control.moveCursor(QtGui.QTextCursor.End)
+
+    def flush_clearoutput(self):
+        """If a clearoutput is pending, execute it."""
+        if self._pending_clearoutput:
+            self._pending_clearoutput = False
+            self.clear_output()
+
+    def clear_output(self):
+        """Clears the current line of output."""
+        cursor = self._control.textCursor()
+        cursor.beginEditBlock()
+        cursor.movePosition(cursor.StartOfLine, cursor.KeepAnchor)
+        cursor.insertText('')
+        cursor.endEditBlock()
+
     #---------------------------------------------------------------------------
     # 'FrontendWidget' protected interface
     #---------------------------------------------------------------------------
