@@ -1,5 +1,6 @@
-""" A FrontendWidget that emulates the interface of the console IPython and
-    supports the additional functionality provided by the IPython kernel.
+"""A FrontendWidget that emulates the interface of the console IPython.
+
+This supports the additional functionality provided by the IPython kernel.
 """
 
 #-----------------------------------------------------------------------------
@@ -22,8 +23,8 @@ from IPython.external.qt import QtCore, QtGui
 from IPython.core.inputsplitter import IPythonInputSplitter
 from IPython.core.inputtransformer import ipy_prompt
 from IPython.utils.traitlets import Bool, Unicode
-from frontend_widget import FrontendWidget
-import styles
+from .frontend_widget import FrontendWidget
+from . import styles
 
 #-----------------------------------------------------------------------------
 # Constants
@@ -134,10 +135,11 @@ class IPythonWidget(FrontendWidget):
         else:
             self.set_default_style()
 
+        self._guiref_loaded = False
+
     #---------------------------------------------------------------------------
     # 'BaseFrontendMixin' abstract interface
     #---------------------------------------------------------------------------
-
     def _handle_complete_reply(self, rep):
         """ Reimplemented to support IPython's improved completion machinery.
         """
@@ -172,11 +174,15 @@ class IPythonWidget(FrontendWidget):
         msg_id = msg['parent_header'].get('msg_id')
         info = self._request_info['execute'].get(msg_id)
         if info and info.kind == 'prompt':
-           number = msg['content']['execution_count'] + 1
-           self._show_interpreter_prompt(number)
-           self._request_info['execute'].pop(msg_id)
+            content = msg['content']
+            if content['status'] == 'aborted':
+                self._show_interpreter_prompt()
+            else:
+                number = content['execution_count'] + 1
+                self._show_interpreter_prompt(number)
+            self._request_info['execute'].pop(msg_id)
         else:
-           super(IPythonWidget, self)._handle_execute_reply(msg)
+            super(IPythonWidget, self)._handle_execute_reply(msg)
 
     def _handle_history_reply(self, msg):
         """ Implemented to handle history tail replies, which are only supported
@@ -216,6 +222,7 @@ class IPythonWidget(FrontendWidget):
         """
         self.log.debug("pyout: %s", msg.get('content', ''))
         if not self._hidden and self._is_from_this_session(msg):
+            self.flush_clearoutput()
             content = msg['content']
             prompt_number = content.get('execution_count', 0)
             data = content['data']
@@ -243,6 +250,7 @@ class IPythonWidget(FrontendWidget):
         # eventually will as this allows all frontends to monitor the display
         # data. But we need to figure out how to handle this in the GUI.
         if not self._hidden and self._is_from_this_session(msg):
+            self.flush_clearoutput()
             source = msg['content']['source']
             data = msg['content']['data']
             metadata = msg['content']['metadata']
@@ -257,10 +265,21 @@ class IPythonWidget(FrontendWidget):
             # This newline seems to be needed for text and html output.
             self._append_plain_text(u'\n', True)
 
+    def _handle_kernel_info_reply(self, rep):
+        """ Handle kernel info replies.
+        """
+        if not self._guiref_loaded:
+            if rep['content'].get('language') == 'python':
+                self._load_guiref_magic()
+            self._guiref_loaded = True
+
     def _started_channels(self):
         """Reimplemented to make a history request and load %guiref."""
         super(IPythonWidget, self)._started_channels()
-        self._load_guiref_magic()
+
+        # The reply will trigger %guiref load provided language=='python'
+        self.kernel_client.kernel_info()
+
         self.kernel_client.shell_channel.history(hist_access_type='tail',
                                                   n=1000)
     
@@ -438,8 +457,8 @@ class IPythonWidget(FrontendWidget):
     def set_default_style(self, colors='lightbg'):
         """ Sets the widget style to the class defaults.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         colors : str, optional (default lightbg)
             Whether to use the default IPython light background or dark
             background or B&W style.
@@ -464,8 +483,8 @@ class IPythonWidget(FrontendWidget):
     def _edit(self, filename, line=None):
         """ Opens a Python script for editing.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         filename : str
             A path to a local system file.
 
@@ -539,7 +558,7 @@ class IPythonWidget(FrontendWidget):
         self.exit_requested.emit(self)
 
     def _handle_payload_next_input(self, item):
-        self.input_buffer = dedent(item['text'].rstrip())
+        self.input_buffer = item['text']
 
     def _handle_payload_page(self, item):
         # Since the plain text widget supports only a very small subset of HTML
