@@ -82,7 +82,6 @@ def pkg_info(pkg_path):
     return dict(
         ipython_version=release.version,
         ipython_path=pkg_path,
-        codename=release.codename,
         commit_source=src,
         commit_hash=hsh,
         sys_version=sys.version,
@@ -93,28 +92,32 @@ def pkg_info(pkg_path):
         default_encoding=encoding.DEFAULT_ENCODING,
         )
 
+def get_sys_info():
+    """Return useful information about IPython and the system, as a dict."""
+    p = os.path
+    path = p.dirname(p.abspath(p.join(__file__, '..')))
+    return pkg_info(path)
 
 @py3compat.doctest_refactor_print
 def sys_info():
     """Return useful information about IPython and the system, as a string.
 
-    Example
-    -------
-    In [2]: print sys_info()
-    {'commit_hash': '144fdae',      # random
-     'commit_source': 'repository',
-     'ipython_path': '/home/fperez/usr/lib/python2.6/site-packages/IPython',
-     'ipython_version': '0.11.dev',
-     'os_name': 'posix',
-     'platform': 'Linux-2.6.35-22-generic-i686-with-Ubuntu-10.10-maverick',
-     'sys_executable': '/usr/bin/python',
-     'sys_platform': 'linux2',
-     'sys_version': '2.6.6 (r266:84292, Sep 15 2010, 15:52:39) \\n[GCC 4.4.5]'}
-   """
-    p = os.path
-    path = p.dirname(p.abspath(p.join(__file__, '..')))
-    return pprint.pformat(pkg_info(path))
-
+    Examples
+    --------
+    ::
+    
+        In [2]: print sys_info()
+        {'commit_hash': '144fdae',      # random
+         'commit_source': 'repository',
+         'ipython_path': '/home/fperez/usr/lib/python2.6/site-packages/IPython',
+         'ipython_version': '0.11.dev',
+         'os_name': 'posix',
+         'platform': 'Linux-2.6.35-22-generic-i686-with-Ubuntu-10.10-maverick',
+         'sys_executable': '/usr/bin/python',
+         'sys_platform': 'linux2',
+         'sys_version': '2.6.6 (r266:84292, Sep 15 2010, 15:52:39) \\n[GCC 4.4.5]'}
+    """
+    return pprint.pformat(get_sys_info())
 
 def _num_cpus_unix():
     """Return the number of active CPUs on a Unix system."""
