@@ -39,6 +39,7 @@ if sys.platform == 'win32':
 else:
     date_format = "%B %-d, %Y"
 
+
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
@@ -109,10 +110,10 @@ class SList(list):
 
     These are normal lists, but with the special attributes:
 
-        .l (or .list) : value as list (the list itself).
-        .n (or .nlstr): value as a string, joined on newlines.
-        .s (or .spstr): value as a string, joined on spaces.
-        .p (or .paths): list of path objects
+    * .l (or .list) : value as list (the list itself).
+    * .n (or .nlstr): value as a string, joined on newlines.
+    * .s (or .spstr): value as a string, joined on spaces.
+    * .p (or .paths): list of path objects
 
     Any values which require transformations are computed only once and
     cached."""
@@ -175,7 +176,7 @@ class SList(list):
             except IndexError:
                 return ""
 
-        if isinstance(pattern, basestring):
+        if isinstance(pattern, py3compat.string_types):
             pred = lambda x : re.search(pattern, x, re.IGNORECASE)
         else:
             pred = pattern
@@ -190,13 +191,14 @@ class SList(list):
         Allows quick awk-like usage of string lists.
 
         Example data (in var a, created by 'a = !ls -l')::
+
             -rwxrwxrwx  1 ville None      18 Dec 14  2006 ChangeLog
             drwxrwxrwx+ 6 ville None       0 Oct 24 18:05 IPython
 
-        a.fields(0) is ['-rwxrwxrwx', 'drwxrwxrwx+']
-        a.fields(1,0) is ['1 -rwxrwxrwx', '6 drwxrwxrwx+']
-        (note the joining by space).
-        a.fields(-1) is ['ChangeLog', 'IPython']
+        * ``a.fields(0)`` is ``['-rwxrwxrwx', 'drwxrwxrwx+']``
+        * ``a.fields(1,0)`` is ``['1 -rwxrwxrwx', '6 drwxrwxrwx+']``
+          (note the joining by space).
+        * ``a.fields(-1)`` is ``['ChangeLog', 'IPython']``
 
         IndexErrors are ignored.
 
@@ -223,6 +225,7 @@ class SList(list):
         """ sort by specified fields (see fields())
 
         Example::
+
             a.sort(1, nums = True)
 
         Sorts a by second field, in numerical order (so that 21 > 3)
@@ -307,7 +310,9 @@ def list_strings(arg):
     """Always return a list of strings, given a string or list of strings
     as input.
 
-    :Examples:
+    Examples
+    --------
+    ::
 
         In [7]: list_strings('A single string')
         Out[7]: ['A single string']
@@ -319,14 +324,16 @@ def list_strings(arg):
         Out[9]: ['A', 'list', 'of', 'strings']
     """
 
-    if isinstance(arg,basestring): return [arg]
+    if isinstance(arg, py3compat.string_types): return [arg]
     else: return arg
 
 
 def marquee(txt='',width=78,mark='*'):
     """Return the input string centered in a 'marquee'.
 
-    :Examples:
+    Examples
+    --------
+    ::
 
         In [16]: marquee('A test',40)
         Out[16]: '**************** A test ****************'
@@ -497,13 +504,14 @@ class EvalFormatter(Formatter):
 
     Examples
     --------
-    
-    In  [1]: f = EvalFormatter()
-    In  [2]: f.format('{n//4}', n=8)
-    Out [2]: '2'
-    
-    In  [3]: f.format("{greeting[slice(2,4)]}", greeting="Hello")
-    Out [3]: 'll'
+    ::
+
+        In [1]: f = EvalFormatter()
+        In [2]: f.format('{n//4}', n=8)
+        Out[2]: '2'
+
+        In [3]: f.format("{greeting[slice(2,4)]}", greeting="Hello")
+        Out[3]: 'll'
     """
     def get_field(self, name, args, kwargs):
         v = eval(name, kwargs)
@@ -525,16 +533,17 @@ class FullEvalFormatter(Formatter):
     
     Examples
     --------
-    
-    In [1]: f = FullEvalFormatter()
-    In [2]: f.format('{n//4}', n=8)
-    Out[2]: u'2'
-    
-    In [3]: f.format('{list(range(5))[2:4]}')
-    Out[3]: u'[2, 3]'
+    ::
 
-    In [4]: f.format('{3*2}')
-    Out[4]: u'6'
+        In [1]: f = FullEvalFormatter()
+        In [2]: f.format('{n//4}', n=8)
+        Out[2]: u'2'
+
+        In [3]: f.format('{list(range(5))[2:4]}')
+        Out[3]: u'[2, 3]'
+
+        In [4]: f.format('{3*2}')
+        Out[4]: u'6'
     """
     # copied from Formatter._vformat with minor changes to allow eval
     # and replace the format_spec code with slicing
@@ -579,15 +588,17 @@ class DollarFormatter(FullEvalFormatter):
 
     Examples
     --------
-    In [1]: f = DollarFormatter()
-    In [2]: f.format('{n//4}', n=8)
-    Out[2]: u'2'
-    
-    In [3]: f.format('23 * 76 is $result', result=23*76)
-    Out[3]: u'23 * 76 is 1748'
-    
-    In [4]: f.format('$a or {b}', a=1, b=2)
-    Out[4]: u'1 or 2'
+    ::
+
+        In [1]: f = DollarFormatter()
+        In [2]: f.format('{n//4}', n=8)
+        Out[2]: u'2'
+
+        In [3]: f.format('23 * 76 is $result', result=23*76)
+        Out[3]: u'23 * 76 is 1748'
+
+        In [4]: f.format('$a or {b}', a=1, b=2)
+        Out[4]: u'1 or 2'
     """
     _dollar_pattern = re.compile("(.*?)\$(\$?[\w\.]+)")
     def parse(self, fmt_string):
@@ -616,14 +627,14 @@ class DollarFormatter(FullEvalFormatter):
 
 def _chunks(l, n):
     """Yield successive n-sized chunks from l."""
-    for i in xrange(0, len(l), n):
+    for i in py3compat.xrange(0, len(l), n):
         yield l[i:i+n]
 
 
 def _find_optimal(rlist , separator_size=2 , displaywidth=80):
     """Calculate optimal info to columnize a list of string"""
     for nrow in range(1, len(rlist)+1) :
-        chk = map(max,_chunks(rlist, nrow))
+        chk = list(map(max,_chunks(rlist, nrow)))
         sumlength = sum(chk)
         ncols = len(chk)
         if sumlength+separator_size*(ncols-1) <= displaywidth :
@@ -650,7 +661,7 @@ def compute_item_matrix(items, empty=None, *args, **kwargs) :
     Parameters
     ----------
 
-    items :
+    items
         list of strings to columize
     empty : (default None)
         default value to fill list if needed
@@ -662,41 +673,43 @@ def compute_item_matrix(items, empty=None, *args, **kwargs) :
     Returns
     -------
 
-    Returns a tuple of (strings_matrix, dict_info)
-
-    strings_matrix :
+    strings_matrix
 
         nested list of string, the outer most list contains as many list as
         rows, the innermost lists have each as many element as colums. If the
         total number of elements in `items` does not equal the product of
         rows*columns, the last element of some lists are filled with `None`.
 
-    dict_info :
+    dict_info
         some info to make columnize easier:
 
-        columns_numbers : number of columns
-        rows_numbers    : number of rows
-        columns_width   : list of with of each columns
-        optimal_separator_width : best separator width between columns
+        columns_numbers
+          number of columns
+        rows_numbers
+          number of rows
+        columns_width
+          list of with of each columns
+        optimal_separator_width
+          best separator width between columns
 
     Examples
     --------
+    ::
 
-    In [1]: l = ['aaa','b','cc','d','eeeee','f','g','h','i','j','k','l']
-       ...: compute_item_matrix(l,displaywidth=12)
-    Out[1]:
-        ([['aaa', 'f', 'k'],
-        ['b', 'g', 'l'],
-        ['cc', 'h', None],
-        ['d', 'i', None],
-        ['eeeee', 'j', None]],
-        {'columns_numbers': 3,
-        'columns_width': [5, 1, 1],
-        'optimal_separator_width': 2,
-        'rows_numbers': 5})
-
+        In [1]: l = ['aaa','b','cc','d','eeeee','f','g','h','i','j','k','l']
+           ...: compute_item_matrix(l,displaywidth=12)
+        Out[1]:
+            ([['aaa', 'f', 'k'],
+            ['b', 'g', 'l'],
+            ['cc', 'h', None],
+            ['d', 'i', None],
+            ['eeeee', 'j', None]],
+            {'columns_numbers': 3,
+            'columns_width': [5, 1, 1],
+            'optimal_separator_width': 2,
+            'rows_numbers': 5})
     """
-    info = _find_optimal(map(len, items), *args, **kwargs)
+    info = _find_optimal(list(map(len, items)), *args, **kwargs)
     nrow, ncol = info['rows_numbers'], info['columns_numbers']
     return ([[ _get_or_default(items, c*nrow+i, default=empty) for c in range(ncol) ] for i in range(nrow) ], info)
 
@@ -725,3 +738,36 @@ def columnize(items, separator='  ', displaywidth=80):
     fmatrix = [filter(None, x) for x in matrix]
     sjoin = lambda x : separator.join([ y.ljust(w, ' ') for y, w in zip(x, info['columns_width'])])
     return '\n'.join(map(sjoin, fmatrix))+'\n'
+
+
+def get_text_list(list_, last_sep=' and ', sep=", ", wrap_item_with=""):
+    """
+    Return a string with a natural enumeration of items
+
+    >>> get_text_list(['a', 'b', 'c', 'd'])
+    'a, b, c and d'
+    >>> get_text_list(['a', 'b', 'c'], ' or ')
+    'a, b or c'
+    >>> get_text_list(['a', 'b', 'c'], ', ')
+    'a, b, c'
+    >>> get_text_list(['a', 'b'], ' or ')
+    'a or b'
+    >>> get_text_list(['a'])
+    'a'
+    >>> get_text_list([])
+    ''
+    >>> get_text_list(['a', 'b'], wrap_item_with="`")
+    '`a` and `b`'
+    >>> get_text_list(['a', 'b', 'c', 'd'], " = ", sep=" + ")
+    'a + b + c = d'
+    """
+    if len(list_) == 0:
+        return ''
+    if wrap_item_with:
+        list_ = ['%s%s%s' % (wrap_item_with, item, wrap_item_with) for
+                 item in list_]
+    if len(list_) == 1:
+        return list_[0]
+    return '%s%s%s' % (
+        sep.join(i for i in list_[:-1]),
+        last_sep, list_[-1])
\ No newline at end of file
